package net.lensfrex.disillusion.client.controller.components.student.offer;

import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.JFXButton;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Text;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.CommunicativeController;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.student.application.ApplicationPostPane;
import net.lensfrex.disillusion.client.data.request.ApplicationRequest;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.data.response.PublicEmployerInfoResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;

@Slf4j
public class StudentOfferCard extends StackPane implements CommunicativeController<StudentOfferCard.InitData> {
    private final OfferResponse offer;
    private PublicEmployerInfoResponse employerInfo;

    private ThreadSafeDialogMaker dialogMaker;

    private String token;

    private static final String[] colors = new String[]{
            "-fx-background-color: #8F3F7E",
            "-fx-background-color: #B5305F",
            "-fx-background-color: #CE584A",
            "-fx-background-color: #DB8D5C",
            "-fx-background-color: #DA854E",
            "-fx-background-color: #E9AB44",
            "-fx-background-color: #99C286",
            "-fx-background-color: #01A05E",
            "-fx-background-color: #4A8895",
            "-fx-background-color: #16669B",
            "-fx-background-color: #2F65A5",
    };

    public StudentOfferCard(OfferResponse offer) {
        this.offer = offer;
    }

    @Override
    public void loadData(InitData initData) {
        this.dialogMaker = initData.dialogMaker;
        this.token = initData.token;
    }

    public void initialize() {
        String color = colors[Math.abs(offer.getJobName().hashCode()) % colors.length];
        header.setStyle(header.getStyle() + color);

        jobName.setText(offer.getJobName());

        getEmployerInfo();
        workingPlaceText.setText("工作地点：" + offer.getPlace());

        if (offer.getSalary() >= 1_0000) {
            salaryText.setText("薪资：" + offer.getSalary() / 1_0000.0 + "w");
        } else {
            salaryText.setText("薪资：" + offer.getSalary() / 1000.0 + "k");
        }

        ComponentFactory.setCardIcon(applicationSendBtn, Constants.Svg.sendSvg);

        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_PROGRESS_URL)
                .addQueryParam("id", offer.getId())
                .failCallback(e -> log.warn("获取offer进度时出现网络问题：" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> log.warn("获取offer进度失败：" + s))
                        .successCallback(s -> Platform.runLater(() -> progressText.setText(s)))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void getEmployerInfo() {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_PUBLIC_INFO_VIEW_URL)
                .addQueryParam("user", offer.getPublisherId())
                .failCallback(e -> log.warn("获取企业信息时发生网络错误：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> log.warn("获取企业信息不成功：\n" + s))
                        .successCallback(s -> {
                            employerInfo = JSONUtil.toBean(s, PublicEmployerInfoResponse.class);
                            Platform.runLater(() -> employerNameText.setText("工作单位：" + employerInfo.getName()));
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    @Data
    @Builder
    @EqualsAndHashCode(callSuper = false)
    public static class InitData extends CommunicativeController.InitData {
        private ThreadSafeDialogMaker dialogMaker;
        private String token;
    }

    @FXML
    private StackPane rootPane;

    @FXML
    private StackPane header;

    @FXML
    private Label jobName;

    @FXML
    private StackPane body;

    @FXML
    private Text progressText;

    @FXML
    private Text salaryText;

    @FXML
    private Text employerNameText;

    @FXML
    private Text workingPlaceText;

    @FXML
    private JFXButton applicationSendBtn;

    @FXML
    void viewDetail(Event event) throws IOException {
        OfferDetailPane detailPage = ComponentFactory.loadComponent("fxml/student/offer/offerDetail.fxml",
                new OfferDetailPane(dialogMaker, this.offer, employerInfo));
        dialogMaker.createDialogWithOneBtn("Offer详情", detailPage);
    }

    @FXML
    void sendApplication(ActionEvent event) {
        try {
            ApplicationPostPane page = ComponentFactory.loadComponent("fxml/student/application/applicationPostPage.fxml",
                    new ApplicationPostPane(this.token));

            dialogMaker.createDialogWithOKAndCancel("发送求职申请", page,
                    e -> this.sendApplication(page.getChooseResumeId())
            );
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void sendApplication(String resumeId) {
        ApplicationRequest request = new ApplicationRequest(offer.getId(), resumeId);
        new OkhttpUtil()
                .url(Constants.ApplicationRequestUrls.APPLICATION_PUBLISH_URL)
                .body(request)
                .addHeader("token", token)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "发送offer申请时出现网络错误"))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "offer申请发送失败：\n" + s))
                        .successCallback(s -> dialogMaker.createMessageDialog("好好好", "offer申请已发送，请等待处理"))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }
}
