package net.lensfrex.disillusion.client.controller.page.student;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.JFXComboBox;
import com.jfoenix.controls.JFXTextField;
import com.jfoenix.effects.JFXDepthManager;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.input.ScrollEvent;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.CommunicativeController;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.student.offer.StudentOfferCard;
import net.lensfrex.disillusion.client.data.ResponseCode;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.Consumer;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.OkHttpClient;
import okhttp3.Response;

import java.util.List;

@Slf4j
public class OfferSquarePage
        extends VBox
        implements CommunicativeController<OfferSquarePage.InitData> {

    private static final OkHttpClient okhttpClient = Constants.globalClient;

    @FXML
    private VBox rootPane;

    @FXML
    private JFXComboBox<String> filterType;

    @FXML
    private JFXTextField filterString;

    @FXML
    private FlowPane offersPane;

    @FXML
    private ScrollPane scrollPane;

    @FXML
    private Label pageLabel;

    private double totalDrag = 0;

    private String token;
    private Pane parentRootPane;

    private ThreadSafeDialogMaker dialogMaker;

    @Override
    public void loadData(InitData data) {
        this.token = data.token;
        this.dialogMaker = data.dialogMaker;

        this.refresh(null);
    }

    @Data
    @Builder
    @EqualsAndHashCode(callSuper = true)
    public static class InitData extends CommunicativeController.InitData {
        private String token;
        private ThreadSafeDialogMaker dialogMaker;
    }

    public void initialize() {
        ObservableList<String> filterList = FXCollections.observableArrayList();
        filterList.add("用人单位");
        filterList.add("职位名称");
        filterList.add("最低薪资");
        filterType.setItems(filterList);
        filterType.getSelectionModel().select(0);

        scrollPane.addEventHandler(ScrollEvent.SCROLL, e -> {
            double downScrollLength = -e.getDeltaY();
            if (downScrollLength < 0) {
                return;
            }

            totalDrag += downScrollLength;
            if (totalDrag > offersPane.getPrefHeight()) {
                refresh(null);
            }
        });
    }

    private int currentPage = 0;

    private static final int DEFAULT_PAGE_SIZE = 5;

    @FXML
    void refresh(ActionEvent event) {
        this.getOffers(currentPage + 1, DEFAULT_PAGE_SIZE);
    }

    private void getOffers(int page, int size) {
        new OkhttpUtil()
                .addQueryParam("page", String.valueOf(page))
                .addQueryParam("size", String.valueOf(size))
                .url(Constants.OfferRequestUrls.GET_ALL_OFFER_URL)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "请求Offer列表时发生了一下错误\n" + e))
                .successCallback(getOfferSuccessCallback)
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private static final OfferResponse[] EMPTY_OFFERS = new OfferResponse[]{};

    private final Consumer<Response> getOfferSuccessCallback = response -> {
        assert response.body() != null;
        String responseString = response.body().string();
        JSONObject responseJson = JSONUtil.parseObj(responseString);

        Integer code = responseJson.getInt("code");
        if (code == null || !code.equals(ResponseCode.SUCCESS.getCode())) {
            dialogMaker.createMessageDialog("啊哈", "获取offer列表时出错了...\n" + responseString);
        }

        JSONObject data = responseJson.getJSONObject("data");

        List<OfferResponse> offers = JSONUtil.toList(data.getJSONArray("result"), OfferResponse.class);

        Platform.runLater(() -> {
            this.showOfferCards(offers);
            currentPage = offers.size() < DEFAULT_PAGE_SIZE ? 0 : data.getInt("page");
        });
    };

    private void showOfferCards(List<OfferResponse> offers) {
        try {
            ObservableList<Node> items = offersPane.getChildren();
            if (currentPage == 0) {
                items.clear();
            }

            for (OfferResponse offer : offers) {
                StudentOfferCard studentOfferCard = new StudentOfferCard(offer);
                Node item = ComponentFactory.loadComponent("fxml/student/offer/offerCard.fxml", studentOfferCard);

                studentOfferCard.loadData(StudentOfferCard.InitData.builder().token(token).dialogMaker(dialogMaker).build());

                studentOfferCard.setUserData("offer");
                JFXDepthManager.setDepth(item, 1);

                items.add(new Pane(item));
            }
        } catch (Exception e) {
            log.error("解析Offer对象数据时出错：\n{}", e.toString());
            e.printStackTrace();
        }
    }

    @FXML
    void search(ActionEvent event) {
        String selected = filterType.getSelectionModel().getSelectedItem();
        String filterType = "";
        switch (selected) {
            case "用人单位":
                filterType = Constants.OfferRequestUrls.OFFER_FILTER_EMPLOYER;
                break;
            case "职位名称":
                filterType = Constants.OfferRequestUrls.OFFER_FILTER_JOB_NAME;
                break;
            case "最低薪资":
                filterType = Constants.OfferRequestUrls.OFFER_FILTER_SALARY;
                break;
        }

        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_SEARCH_URL)
                .addQueryParam("filter", filterType)
                .addQueryParam("value", filterString.getText())
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "搜索offer时发生了错误：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "搜索offer不成功：\n" + s))
                        .successCallback(s -> Platform.runLater(() -> {
                            offersPane.getChildren().clear();
                            List<OfferResponse> offers = JSONUtil.toList(s, OfferResponse.class);
                            this.showOfferCards(offers);
                        }))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }
}
