package net.lensfrex.disillusion.client.controller.page.student;

import cn.hutool.core.codec.Base64;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jfoenix.effects.JFXDepthManager;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.VBox;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.CommunicativeController;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.student.resume.ResumeCard;
import net.lensfrex.disillusion.client.controller.components.student.resume.ResumeUploadPane;
import net.lensfrex.disillusion.client.data.ResponseCode;
import net.lensfrex.disillusion.client.data.request.ResumeRequest;
import net.lensfrex.disillusion.client.data.response.ResumeResponse;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.*;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.function.Consumer;

@Slf4j
public class ResumeManagePage extends VBox
        implements CommunicativeController<ResumeManagePage.InitData> {

    private static final OkHttpClient okhttpClient = Constants.globalClient;

    private String token;
    private ThreadSafeDialogMaker dialogMaker;

    @Override
    public void loadData(InitData initData) {
        this.token = initData.token;
        this.dialogMaker = initData.dialogMaker;
        this.refreshResumeList();
    }

    @Data
    @EqualsAndHashCode(callSuper = false)
    public static class InitData extends CommunicativeController.InitData {
        private String token;
        private ThreadSafeDialogMaker dialogMaker;
    }

    public void initialize() {
        resumeContainer.getChildren().addAll(resumeList);
    }

    @FXML
    private VBox rootPane;

    @FXML
    private FlowPane resumeContainer;

    private final ObservableList<ResumeCard> resumeList = FXCollections.observableArrayList();

    @FXML
    void addResume(ActionEvent event) {
        try {
            ResumeUploadPane resumeUploadPane =
                    ComponentFactory.loadComponent("fxml/student/resume/resumeUpload.fxml", new ResumeUploadPane(dialogMaker));

            dialogMaker.createDialogWithOKAndCancel("上传新的简历", resumeUploadPane, e -> {
                try {
                    ResumeRequest request = new ResumeRequest();
                    request.setTitle(resumeUploadPane.getResumeName());
                    request.setRemark(resumeUploadPane.getRemark());

                    File resumeFile = resumeUploadPane.getResumeFile();
                    if (resumeFile == null) {
                        dialogMaker.createMessageDialog("啊哈", "请选择上传一个文件");
                        return;
                    }
                    request.setResumeFileBase64(Base64.encode(resumeFile));

                    this.uploadResume(request);
                } catch (Exception ex) {
                    ex.printStackTrace();
                    dialogMaker.createMessageDialog("啊哈", "读取简历文件时发生错误...\n" + ex);
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void uploadResume(ResumeRequest resumeRequest) {
        String json = JSONUtil.toJsonStr(resumeRequest);
        RequestBody requestBody = RequestBody.create(json, MediaType.get("application/json"));

        Request uploadRequest = new Request.Builder()
                .url(Constants.ResumeRequestUrls.RESUME_UPLOAD_URL)
                .addHeader("token", token)
                .post(requestBody)
                .build();

        okhttpClient.newCall(uploadRequest).enqueue(resumeUploadCallback);
    }

    private final Callback resumeUploadCallback = new Callback() {
        @Override
        public void onFailure(@NotNull Call call, @NotNull IOException e) {
            dialogMaker.createMessageDialog("啊哈", "上传简历时发生了网络错误\n" + e);
        }

        @Override
        public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
            assert response.body() != null;
            String responseString = response.body().string();
            JSONObject responseJson = JSONUtil.parseObj(responseString);

            Integer code = responseJson.getInt("code");
            if (code == null || !code.equals(ResponseCode.SUCCESS.getCode())) {
                String message = responseJson.getStr("message");
                dialogMaker.createMessageDialog("啊哈", "获取用户信息时出错了...\n" + message);
            } else {
                dialogMaker.createMessageDialog("好耶", "上传成功");
                Platform.runLater(ResumeManagePage.this::refreshResumeList);
            }
        }
    };

    private void refreshResumeList() {
        Request uploadRequest = new Request.Builder()
                .url(Constants.ResumeRequestUrls.RESUME_GET_ALL_URL)
                .addHeader("token", token)
                .get()
                .build();

        okhttpClient.newCall(uploadRequest).enqueue(resumeFetchCallback);
    }

    private final Callback resumeFetchCallback = new Callback() {
        @Override
        public void onFailure(@NotNull Call call, @NotNull IOException e) {
            dialogMaker.createMessageDialog("啊哈", "获取简历时发生了网络错误\n" + e);
        }

        @Override
        public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
            assert response.body() != null;
            String responseString = response.body().string();
            JSONObject responseJson = JSONUtil.parseObj(responseString);

            Integer code = responseJson.getInt("code");
            if (code == null || !code.equals(ResponseCode.SUCCESS.getCode())) {
                String message = responseJson.getStr("message");
                dialogMaker.createMessageDialog("啊哈", "获取简历时出错了...\n" + message);
            }

            List<ResumeResponse> result = JSONUtil.toList(responseJson.getJSONArray("data"), ResumeResponse.class);
            Platform.runLater(() -> listResumes(result));
        }
    };

    private void listResumes(List<ResumeResponse> resumes) {
        try {
            ObservableList<Node> items = resumeContainer.getChildren();
            items.clear();
            for (ResumeResponse resume : resumes) {
                if (resume == null) {
                    continue;
                }

                ResumeCard itemController = new ResumeCard(resume, token, dialogMaker, updateCallback, deleteCallback);
                ResumeCard item =
                        ComponentFactory.loadComponent("fxml/student/resume/resumeCard.fxml", itemController);
                JFXDepthManager.setDepth(item, 1);
                item.setUserData(resume);
                items.add(item);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.warn("展示简历卡片时发生错误");
        }
    }

    private final Consumer<Response> updateCallback = response -> refreshResumeList();

    private final Consumer<Response> deleteCallback = response -> refreshResumeList();
}
