package net.lensfrex.disillusion.client.controller.page.student;

import animatefx.animation.FadeIn;
import animatefx.animation.FadeOut;
import cn.hutool.json.JSONUtil;
import javafx.application.Platform;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.image.ImageView;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Text;
import lombok.Builder;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.response.StudentInfoResponse;
import net.lensfrex.disillusion.client.util.DialogMaker;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.*;

import java.util.HashMap;
import java.util.Map;

@Slf4j
public class StudentMainController {
    private static final OkHttpClient okhttpClient = Constants.globalClient;

    @FXML
    private StackPane rootPane;

    @FXML
    private ImageView avatarImageView;

    @FXML
    private Text usernameText;

    @FXML
    private Text realNameText;

    @FXML
    private StackPane containerPane;

    @FXML
    private Text signText;

    private ThreadSafeDialogMaker threadSafeDialogMaker;
    private DialogMaker dialogMaker;

    private String token;
    private String username;

    private StudentInfoResponse studentInfo;

    private final Map<String, Pane> pages = new HashMap<>(4);

    public void initialize() {
        threadSafeDialogMaker = new ThreadSafeDialogMaker(rootPane);
        dialogMaker = new DialogMaker(rootPane);
        try {
            pages.put("offerSquare", ComponentFactory.loadComponent("fxml/student/offer/offerSquare.fxml", OfferSquarePage.class));
            pages.put("profile", ComponentFactory.loadComponent("fxml/student/profile.fxml", StudentProfilePage.class));
            pages.put("applicationManage", ComponentFactory.loadComponent("fxml/student/application/applicationManagePage.fxml", ApplicationManagePage.class));
            pages.put("resumeManage", ComponentFactory.loadComponent("fxml/student/resume/resumeManagePage.fxml", ResumeManagePage.class));
        } catch (Exception e) {
            e.printStackTrace();
            DialogMaker.errorMessageDialog("出错啦，请联系lensfrex这家伙\n调试信息：\n" + e);
        }
    }

    public void setData(InitData data) {
        this.token = data.token;
        this.username = data.username;

        this.updateUserInfo();
    }

    private void updateUserInfo() {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_INFO_VIEW_URL)
                .token(token)
                .failCallback(e -> log.error("请求用户数据时发生网络错误"))
                .successCallback(response -> {
                    ResponseHandler.Result result = ResponseHandler.parse(response,
                                    s -> threadSafeDialogMaker.createMessageDialog("啊哈", "获取用户信息时出错：" + s));
                    if (!result.success) {
                        return;
                    }

                    studentInfo = JSONUtil.toBean(result.data, StudentInfoResponse.class);
                    Platform.runLater(() -> {
                        usernameText.setText(username);
                        realNameText.textProperty().set("@" + studentInfo.getRealName());
                        signText.setText(studentInfo.getShortDescribe());
                    });

                    new OkhttpUtil()
                            .url(Constants.UserRequestUrls.USER_AVATAR_VIEW_URL)
                            .token(token)
                            .successCallback(avatarResponse -> Platform.runLater(() -> StudentProfilePage.setAvatar(avatarImageView, avatarResponse)))
                            .executeAsync(OkhttpUtil.RequestMethod.GET);
                })
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    @FXML
    void showApplications(ActionEvent event) {
        ApplicationManagePage controller = (ApplicationManagePage) this.switchPage("applicationManage");
        controller.loadData(ApplicationManagePage.InitData.builder()
                .token(token)
                .dialogMaker(threadSafeDialogMaker)
                .build()
        );
    }

    @FXML
    void showOfferSquare(ActionEvent event) {
        OfferSquarePage controller = (OfferSquarePage) this.switchPage("offerSquare");
        controller.loadData(OfferSquarePage.InitData.builder()
                .token(token)
                .dialogMaker(threadSafeDialogMaker)
                .build()
        );
    }

    @FXML
    void showProfile(ActionEvent event) {
        StudentProfilePage controller = (StudentProfilePage) this.switchPage("profile");
        controller.setData(studentInfo, token, threadSafeDialogMaker, info -> this.updateUserInfo());
    }

    @FXML
    void showResumeManage(ActionEvent event) {
        ResumeManagePage controller = (ResumeManagePage) this.switchPage("resumeManage");
        ResumeManagePage.InitData data = new ResumeManagePage.InitData();
        data.setToken(token);
        data.setDialogMaker(threadSafeDialogMaker);

        controller.loadData(data);
    }

    private String currentWindow = "";

    private Node switchPage(String page) {
        if (currentWindow.equals(page)) {
            return pages.get(page);
        }

        ObservableList<Node> list = containerPane.getChildren();

        new FadeOut(list.get(0)).play();
        Pane pagePane = pages.get(page);
        if (pagePane != null) {
            list.clear();
            list.add(pagePane);
            new FadeIn(pagePane).play();
            currentWindow = page;
        }

        return pagePane;
    }

    @Data
    @Builder
    public static class InitData {
        private String token;
        private String username;
    }
}
