package net.lensfrex.disillusion.client.util;

import cn.hutool.core.util.URLUtil;
import cn.hutool.json.JSONUtil;
import net.lensfrex.disillusion.client.config.Constants;
import okhttp3.*;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

public class OkhttpUtil {
    private static final OkHttpClient client = Constants.globalClient;

    private Request request;
    private RequestBody requestBody;

    private final Request.Builder requestBuilder;
    private FormBody.Builder formBuilder;

    private Consumer<Response> successCallback;
    private java.util.function.Consumer<IOException> failCallback;

    private String url;

    private final Callback callbackWrapper = new Callback() {
        @Override
        public void onFailure(@NotNull Call call, @NotNull IOException e) {
            try {
                failCallback.accept(e);
            } catch (Exception ex) {
                e.printStackTrace();
            }
        }

        @Override
        public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
            try {
                successCallback.accept(response);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    public OkhttpUtil() {
        requestBuilder = new Request.Builder();
    }

    public OkhttpUtil url(String url) {
        this.url = url;
        return this;
    }

    private Map<String, String> queryParams;

    public OkhttpUtil addQueryParam(String key, String value) {
        if (queryParams == null) {
            queryParams = new HashMap<>(10);
        }

        queryParams.put(key, value);

        return this;
    }

    public OkhttpUtil token(String token) {
        return this.addHeader("token", token);
    }

    public OkhttpUtil addHeader(String key, String value) {
        requestBuilder.addHeader(key, value);

        return this;
    }

    public OkhttpUtil successCallback(Consumer<Response> successCallback) {
        this.successCallback = successCallback;

        return this;
    }

    public OkhttpUtil failCallback(java.util.function.Consumer<IOException> failCallbackCallback) {
        this.failCallback = failCallbackCallback;

        return this;
    }

    public OkhttpUtil callback(Consumer<Response> successCallback, java.util.function.Consumer<IOException> failCallbackCallback) {
        this.successCallback = successCallback;
        this.failCallback = failCallbackCallback;

        return this;
    }

    public OkhttpUtil form(String key, String value) {
        if (formBuilder == null) {
            formBuilder = new FormBody.Builder();
        }

        formBuilder.add(key, value);

        return this;
    }

    public OkhttpUtil body(String data, String contentType) {
        requestBody = RequestBody.create(data, MediaType.parse(contentType));

        return this;
    }

    public OkhttpUtil body(Object data) {
        String jsonData = JSONUtil.toJsonStr(data);
        return body(jsonData, "application/json");
    }

    public OkhttpUtil body(byte[] data, String contentType) {
        requestBody = RequestBody.create(data, MediaType.parse(contentType));

        return this;
    }

    private static final byte[] EMPTY_DATA = "".getBytes(StandardCharsets.UTF_8);

    public void executeAsync(RequestMethod method) {
        this.execute(method).enqueue(callbackWrapper);
    }

    public Response executeSync(RequestMethod method) {
        try {
            return execute(method).execute();
        } catch (IOException e) {
            if (failCallback != null) {
                failCallback.accept(e);
            }
        }

        return null;
    }

    private Call execute(RequestMethod method) {
        if (queryParams == null) {
            requestBuilder.url(URLUtil.url(url));
        } else {
            String queryParamString = URLUtil.buildQuery(queryParams, StandardCharsets.UTF_8);
            requestBuilder.url(URLUtil.url(url + "?" + queryParamString));
        }

        switch (method) {
            case GET:
                request = requestBuilder.get().build();
                return client.newCall(request);
            case POST:
                if (requestBody == null) {
                    if (formBuilder == null) {
                        requestBody = RequestBody.create(EMPTY_DATA);
                    } else {
                        requestBody = formBuilder.build();
                    }
                }
                request = requestBuilder.post(requestBody).build();
                return client.newCall(request);
            default:
                throw new IllegalArgumentException();
        }
    }

    public static enum RequestMethod {
        GET, POST
    }
}