package net.lensfrex.disillusion.dao;

import net.lensfrex.disillusion.configure.GlobalConfigure;
import net.lensfrex.disillusion.dao.database.Table;
import net.lensfrex.disillusion.dao.database.VirtualDatabase;
import net.lensfrex.disillusion.data.Hash;
import net.lensfrex.disillusion.data.List;
import net.lensfrex.disillusion.model.StoreData;

import java.io.IOException;

public abstract class DaoBase<T extends StoreData> implements Table<T> {
    // 表名，如"resume", "offer", "user_info"等
    private final String tableName;

    public DaoBase(GlobalConfigure globalConfigure, String tableName)
            throws IOException, ClassNotFoundException {

        this.tableName = tableName;
        VirtualDatabase.init(globalConfigure.getDatabaseDir());
        updateData();
    }

    // 存放实际数据Hash表，泛型T需要符合StoreData接口，即具有id字段
    // 此时id作为hash表的key进行存取
    private Hash<String, T> table;

    @Override
    public T select(String id) throws IOException, ClassNotFoundException {
        updateData();
        return table.get(id);
    }

    /**
     * 获取所有数据
     */
    public List<T> getAll() throws IOException, ClassNotFoundException {
        updateData();
        return table.allValues();
    }

    @Override
    public void delete(T record) throws IOException, ClassNotFoundException {
        updateData();
        delete(record.getId());
    }

    /**
     * 按照id删除一条数据
     */
    @Override
    public void delete(String id) throws IOException, ClassNotFoundException {
        updateData();
        table.delete(id);
        VirtualDatabase.saveTable(tableName, table);
    }

    /**
     * 插入或者更新一条数据，当数据记录不存在时则新增，否则更新该条数据
     */
    @Override
    public void insertOrUpdate(T record) throws IOException, ClassNotFoundException {
        updateData();
        table.put(record.getId(), record);
        VirtualDatabase.saveTable(tableName, table);
    }

    @Override
    public void update(T record) throws IOException, ClassNotFoundException {
        updateData();
        table.put(record.getId(), record);
        VirtualDatabase.saveTable(tableName, table);
    }

    @SuppressWarnings("unchecked")
    private void updateData() throws IOException, ClassNotFoundException {
        table = (Hash<String, T>) VirtualDatabase.getTable(tableName, Hash.class);
        if (table == null) {
            table = new Hash<>();
            VirtualDatabase.saveTable(tableName, table);
        }
    }
}
