package net.lensfrex.disillusion.web.controller;

import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.OfferApplication;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.employer.Offer;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.TokenUtil;
import net.lensfrex.disillusion.web.response.OfferListResponse;
import net.lensfrex.disillusion.web.service.employer.OfferService;
import net.lensfrex.disillusion.web.service.student.ApplicationService;
import net.lensfrex.disillusion.web.service.user.UserService;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.Arrays;

@RestController
@RequestMapping("/offer")
public class OfferController {

    private final OfferService offerService;
    private final UserService userService;
    private final ApplicationService applicationService;

    public OfferController(OfferService offerService, UserService userService,
                           ApplicationService applicationService) {

        this.offerService = offerService;
        this.userService = userService;
        this.applicationService = applicationService;
    }

    private static final String OFFER_FILTER_EMPLOYER = "by_employer";
    private static final String OFFER_FILTER_JOB_NAME = "by_job_name";
    private static final String OFFER_FILTER_SALARY = "by_salary";

    @GetMapping("/search")
    public Response<Offer[]> searchOffer(@RequestParam("filter") String filter, @RequestParam("value") String filterValue)
            throws IOException, ClassNotFoundException {

        Offer[] result = switch (filter) {
            case OFFER_FILTER_EMPLOYER -> offerService.searchOfferByUser(filterValue);
            case OFFER_FILTER_JOB_NAME -> offerService.searchOfferByName(filterValue);
            case OFFER_FILTER_SALARY -> offerService.getAllOfferBySalary(filterValue);
            default -> throw new IllegalStateException("Unexpected value: " + filter);
        };

        return Response.success(result);
    }

    private static final OfferListResponse.OfferResponse[] EMPTY_OFFER = new OfferListResponse.OfferResponse[]{};

    @GetMapping("/getAll")
    public Response<OfferListResponse> getOffer(@RequestParam("page") Integer page, @RequestParam("size") Integer size)
            throws IOException, ClassNotFoundException {
        Offer[] allOffer = offerService.getAllOffer();

        int start = (page - 1) * size;
        if (start > allOffer.length) {
            OfferListResponse response = new OfferListResponse();
            response.setTotal(0);
            response.setPage(0);
            response.setSize(0);
            response.setResult(EMPTY_OFFER);

            return Response.success(response);
        }

        Offer[] offers = Arrays.copyOfRange(allOffer, start, Math.min(start + size, allOffer.length));
        OfferListResponse response = new OfferListResponse();
        OfferListResponse.OfferResponse[] offerList = new OfferListResponse.OfferResponse[offers.length];
        for (int i = 0; i < offers.length; i++) {
            Offer offer = offers[i];
            offerList[i] = new OfferListResponse.OfferResponse();
            BeanUtils.copyProperties(offer, offerList[i]);
        }

        response.setTotal(allOffer.length);
        response.setPage(page);
        response.setSize(size);
        response.setResult(offerList);

        return Response.success(response);
    }

    @GetMapping("/get")
    public Response<OfferListResponse.OfferResponse> getOffer(@RequestParam("id") String id)
            throws IOException, ClassNotFoundException {

        Offer offer = offerService.getOffer(id);
        if (offer == null) {
            return Response.success();
        }

        OfferListResponse.OfferResponse response = new OfferListResponse.OfferResponse();
        User publisher = userService.getAccount(offer.getPublisherId());
        if (publisher == null) {
            return Response.success();
        }

        BeanUtils.copyProperties(offer, response);

        return Response.success(response);
    }

    @GetMapping("/progress")
    public Response<String> getStatus(@RequestParam("id") String id)
            throws IOException, ClassNotFoundException {
        Offer offer = offerService.getOffer(id);
        if (offer == null) {
            return Response.success("unknown");
        }

        int target = offer.getTargetNumber();

        int count = 0;

        OfferApplication[] applications = applicationService.getAllApplicationByOfferId(id);
        for (OfferApplication application : applications) {
            if (application == null) {
                continue;
            }
            if (application.getStatus() == OfferApplication.Status.ACCEPTED) {
                count++;
            }
        }

        return Response.success(count + "/" + target);
    }

    private static final String samples = """
              8．微专业课程选课要求：

              前期经过自愿报名并审核成功的学生，根据所报微专业的培养方案，自行选择相应课程。

              9．认真对照培养方案，选课时务必仔细核对课程信息，确保“课程代码”、“课程名称”、“学分”这三项重要信息与培养方案要求相符。

              10．禁止同一时间修读两门及以上课程，系统会提示时间冲突并禁止选课。提前了解专业教学进程（实习、实践、课程设计等）情况，不能选择与本专业其它教学任务周次有冲突的课堂。

              11．因培养方案变动、转专业等原因，计划修读的课程确定不再开课的，请先查询本科生课程学分认定对照表并咨询开课学院，再进行选课。待课程修读完获得学分后，再按照本科生院发布的相关要求办理学分认定。

              12．学生选课、听课、考核必须一致。学生不能参加未选择课程的学习和考核，即使参加了学习和考核，如果没有选课，也是无效的，将不记录成绩。

              13．学生凭学号、密码选课，认真对待，并对自己的选课行为负责。密码必须妥善保管，不得代替他人选课，不得借用、盗用他人学号及密码选课。

              14．学生登录本科教学综合管理系统需通过学校统一身份认证平台登录，登录方式、密码重置方式详见以下两个通知：https://jwc.wust.edu.cn/2022/0226/c1925a255654/page.htm、https://jwc.wust.edu.cn/2022/0302/c1925a255786/page.htm。

            对账号、密码有疑问的，请按照登录页面上“常见问题”、“忘记密码”等提示进行操作。

              15．本次选课为2022-2023学年第二学期课程的最后一次选课，请同学们严格按照时间安排进行退改选，尤其要在规定的时间内查看课表，以免出现课堂因人数不足取消而本人又没有及时参加补选，再次造成某些课程漏选的情况。无故不参加选课或错过选课机会，后果自负。\s

            本科生院教务管理办公室

            二〇二三年二月二十三日""";
}
