package net.lensfrex.disillusion.web.controller.user;

import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.UserInfo;
import net.lensfrex.disillusion.model.employer.Employer;
import net.lensfrex.disillusion.model.student.Student;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.TokenUtil;
import net.lensfrex.disillusion.web.service.user.UserInfoService;
import net.lensfrex.disillusion.web.service.user.UserService;
import org.mindrot.jbcrypt.BCrypt;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/user")
public class UserController {
    private final UserService userService;
    private final UserInfoService userInfoService;

    public UserController(UserService userService,
                          UserInfoService userInfoService) {

        this.userService = userService;
        this.userInfoService = userInfoService;
    }

    @PostMapping(value = "/login", produces = "application/json;charset=utf-8")
    public Response<Map<String, String>> login(@RequestParam("username") String username,
                               @RequestParam("password") String password)
            throws IOException, ClassNotFoundException {

        User user = userService.getAccount(username);
        if (user == null || !BCrypt.checkpw(password, user.getPassword())) {
            return Response.error(ResponseCode.IDENTIFY_ERROR);
        }

        String token = TokenUtil.signNewToken(username);

        Map<String, String> responseData = new HashMap<>(2);
        responseData.put("token", token);
        responseData.put("user_type", user.getUserType().code + "");

        return Response.success(responseData);
    }

    @CheckAuth
    @PostMapping(value = "/refresh", produces = "application/json;charset=utf-8")
    public Response<String> refreshToken(@RequestHeader("token") String token) throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (user == null) {
            return Response.error(ResponseCode.USER_NOT_EXISTS);
        }

        String newToken = TokenUtil.signNewToken(username);

        return Response.success(newToken);
    }

    @PostMapping(value = "/register", produces = "application/json;charset=utf-8")
    public Response<Object> register(@RequestParam("username") String username,
                                     @RequestParam("password") String password,
                                     @RequestParam("userType") String userType)
            throws IOException, ClassNotFoundException {

        // 检查用户是否已经存在
        User user = userService.getAccount(username);
        if (user != null) {
            return Response.error(ResponseCode.USER_ALREADY_EXISTS);
        }

        // 注册用户类型
        User.UserType enumUserType = getUserType(userType);
        if (enumUserType == null) {
            return Response.error(ResponseCode.PARAM_WRONG);
        }

        // 注册用户
        userService.registerNewUser(username, password, enumUserType);

        // 生成默认的用户信息
        userInfoService.saveUserInfoById(getDefaultUserInfo(username, enumUserType));

        return Response.success();
    }

    private User.UserType getUserType(String value) {
        return switch (value) {
            case "0" -> User.UserType.STUDENT;
            case "1" -> User.UserType.EMPLOYER;
            default -> null;
        };
    }

    private UserInfo getDefaultUserInfo(String username, User.UserType userType) {
        try {
            File avatarBaseDir = new File(UserInfoController.AVATAR_BASE_DIR);
            if (!avatarBaseDir.exists()) {
                avatarBaseDir.mkdirs();
            }

            Files.write(Paths.get(UserInfoController.AVATAR_BASE_DIR, username), Student.DEFAULT_AVATAR,
                    StandardOpenOption.CREATE, StandardOpenOption.WRITE);

        } catch (IOException e) {
            log.error("写入默认头像时发生异常", e);
        }

        return switch (userType) {
            case STUDENT -> Student.getDefaultInfo(username);
            case EMPLOYER -> Employer.getDefaultInfo(username);
        };
    }
}
