package net.lensfrex.disillusion.web.controller.user;

import cn.hutool.core.codec.Base64;
import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.UserInfo;
import net.lensfrex.disillusion.web.response.user.PublicEmployerInfoResponse;
import net.lensfrex.disillusion.web.response.user.PublicStudentInfoResponse;
import net.lensfrex.disillusion.web.response.user.UserInfoResponseBase;
import net.lensfrex.disillusion.model.employer.Employer;
import net.lensfrex.disillusion.model.student.Student;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.IOUtil;
import net.lensfrex.disillusion.web.service.user.UserInfoService;
import net.lensfrex.disillusion.web.service.user.UserService;
import net.lensfrex.disillusion.util.TokenUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

@Slf4j
@RestController
@RequestMapping("/user/info")
public class UserInfoController {
    public static final String AVATAR_BASE_DIR = "database/files/avatar";

    private static final ObjectMapper objectMapper = new ObjectMapper();

    private final UserService userService;
    private final UserInfoService userInfoService;

    public UserInfoController(UserService userService,
                              UserInfoService userInfoService) {

        this.userService = userService;
        this.userInfoService = userInfoService;
    }

    // 获取自己的用户信息
    @CheckAuth
    @GetMapping(value = "/view", produces = "application/json;charset=utf-8")
    public Response<UserInfo> getUserInfo(@RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (user == null) {
            return Response.error(ResponseCode.USER_NOT_EXISTS);
        }

        UserInfo info = userInfoService.getUserInfoById(user.getId());
        return Response.success(info);
    }

    @GetMapping(value = "/view/public", produces = "application/json")
    public Response<UserInfoResponseBase> getUserInfoPublic(@RequestParam("user") String username)
            throws IOException, ClassNotFoundException {

        User user = userService.getAccount(username);
        if (user == null) {
            return Response.error(ResponseCode.USER_NOT_EXISTS);
        }

        UserInfo info = userInfoService.getUserInfoById(username);

        return switch (user.getUserType()) {
            case STUDENT -> {
                PublicStudentInfoResponse response = new PublicStudentInfoResponse();
                BeanUtils.copyProperties(info, response);

                yield Response.success(response);
            }
            case EMPLOYER -> {
                PublicEmployerInfoResponse response = new PublicEmployerInfoResponse();
                BeanUtils.copyProperties(info, response);

                yield Response.success(response);
            }
        };
    }

    @CheckAuth
    @PostMapping(value = "/edit", produces = "application/json;charset=utf-8")
    public Response<Object> editUserInfo(@RequestHeader("token") String token, @RequestBody String infoJson)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (user == null) {
            return Response.error(ResponseCode.USER_NOT_EXISTS);
        }

        try {
            UserInfo userInfo = switch (user.getUserType()) {
                case STUDENT -> objectMapper.readValue(infoJson, Student.class);
                case EMPLOYER -> objectMapper.readValue(infoJson, Employer.class);
            };

            userInfo.setId(user.getId());
            userInfoService.saveUserInfoById(userInfo);

            return Response.success();

        } catch (JsonMappingException | JsonParseException e) {
            return Response.error(ResponseCode.PARAM_WRONG);

        } catch (Exception e) {
            return Response.error(ResponseCode.SERVER_INTERNAL_ERROR);
        }
    }

    @GetMapping(value = "/avatar", produces = "image/png")
    public byte[] getAvatarImage(@RequestParam(value = "user", required = false) String user,
                                           @RequestHeader(value = "token", required = false) String token)
             throws IOException, ClassNotFoundException {
        if (user == null && token == null) {
            return Response.error(ResponseCode.PARAM_WRONG).toBytes();
        }

        if (user == null) {
            String username = TokenUtil.getUsernameFromToken(token);
            User userData = userService.getAccount(username);
            if (userData == null) {
                return Response.error(ResponseCode.USER_NOT_EXISTS).toBytes();
            }

            return Files.readAllBytes(Paths.get(AVATAR_BASE_DIR, username));
        } else {
            return Files.readAllBytes(Paths.get(AVATAR_BASE_DIR, user));
        }
    }

    @CheckAuth
    @PostMapping(value = "/avatar/upload", produces = "application/json")
    public Response<Object> uploadAvatar(@RequestHeader(value = "token") String token, @RequestBody String avatarBase64)
             throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (user == null) {
            return Response.error(ResponseCode.USER_NOT_EXISTS);
        }

        try {
            File avatarBaseDir = new File(UserInfoController.AVATAR_BASE_DIR);
            if (!avatarBaseDir.exists()) {
                avatarBaseDir.mkdirs();
            }

            Files.write(Paths.get(UserInfoController.AVATAR_BASE_DIR, username), Base64.decode(avatarBase64),
                    StandardOpenOption.CREATE, StandardOpenOption.WRITE);
        } catch (Exception e) {
            return Response.error(ResponseCode.SERVER_INTERNAL_ERROR);
        }

        return Response.success();
    }
}
