package net.lensfrex.disillusion.web.service.employer;

import cn.hutool.core.lang.Validator;
import net.lensfrex.disillusion.dao.OfferApplicationTable;
import net.lensfrex.disillusion.dao.OfferTable;
import net.lensfrex.disillusion.dao.index.EmployerOfferIndex;
import net.lensfrex.disillusion.dao.index.application.OfferApplicationIndex;
import net.lensfrex.disillusion.data.ArrayList;
import net.lensfrex.disillusion.data.List;
import net.lensfrex.disillusion.model.MultiIndex;
import net.lensfrex.disillusion.model.employer.Employer;
import net.lensfrex.disillusion.model.employer.Offer;
import net.lensfrex.disillusion.web.request.OfferPublishRequest;
import net.lensfrex.disillusion.web.service.user.UserInfoService;
import org.apache.commons.lang.RandomStringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Arrays;

@Service
public class OfferService {
    private final EmployerOfferIndex employerOfferIndex;
    private final OfferTable offerTable;
    private final UserInfoService userInfoService;

    private final OfferApplicationIndex offerApplicationIndex;
    private final OfferApplicationTable offerApplicationTable;

    public OfferService(EmployerOfferIndex EmployerOfferIndex,
                        OfferTable offerTable, UserInfoService userInfoService,
                        OfferApplicationIndex offerApplicationIndex,
                        OfferApplicationTable offerApplicationTable) {

        this.employerOfferIndex = EmployerOfferIndex;
        this.offerTable = offerTable;
        this.userInfoService = userInfoService;
        this.offerApplicationIndex = offerApplicationIndex;
        this.offerApplicationTable = offerApplicationTable;
    }

    public void addOffer(String user, OfferPublishRequest request) throws IOException, ClassNotFoundException {
        Offer offer = packageOffer(user, RandomStringUtils.randomAlphanumeric(8), request);

        MultiIndex multiIndex = employerOfferIndex.select(user);
        if (multiIndex == null) {
            multiIndex = new MultiIndex(user);
            multiIndex.addValue(offer.getId());
        } else {
            multiIndex.addValue(offer.getId());
        }

        offerTable.insertOrUpdate(offer);
        employerOfferIndex.insertOrUpdate(multiIndex);
    }

    public void updateOffer(String user, String offerId, OfferPublishRequest request) throws IOException, ClassNotFoundException {
        MultiIndex multiIndex = employerOfferIndex.select(user);
        if (multiIndex == null || !multiIndex.hasValue(offerId)) {
            return;
        }

        offerTable.insertOrUpdate(packageOffer(user, offerId, request));
    }

    private Offer packageOffer(String user, String id, OfferPublishRequest request) {
        Offer offer = new Offer();
        offer.setPublisherId(user);
        offer.setId(id);

        BeanUtils.copyProperties(request, offer);
        return offer;
    }

    public Offer[] getAllOfferByUser(String user) throws IOException, ClassNotFoundException {
        MultiIndex index = employerOfferIndex.select(user);
        if (index == null) {
            return new Offer[0];
        }

        String[] ids = index.getAllValues();
        List<Offer> offers = new ArrayList<>(ids.length);
        for (String id : ids) {
            if (id == null) {
                continue;
            }

            offers.add(offerTable.select(id));
        }

        return offers.toArray(new Offer[offers.size()]);
    }

    public Offer[] searchOfferByUser(String keyWord) throws IOException, ClassNotFoundException {
        List<Offer> offers = new ArrayList<>(32);
        List<Employer> employers = userInfoService.getAllEmployerUserInfo();

        for (int i = 0; i < employers.size(); i++) {
            Employer employer = employers.get(i);
            if (employer.getName().matches(".*?" + keyWord + ".*")) {
                MultiIndex index = employerOfferIndex.select(employer.getId());
                if (index == null) {
                    return new Offer[0];
                }

                String[] ids = index.getAllValues();
                for (String id : ids) {
                    offers.add(offerTable.select(id));
                }
            }
        }

        return offers.toArray(new Offer[offers.size()]);
    }

    public Offer[] getAllOfferBySalary(String minSalaryText) throws IOException, ClassNotFoundException {
        if (!Validator.isNumber(minSalaryText)) {
            return new Offer[0];
        }

        int minSalary = Integer.parseInt(minSalaryText);
        List<Offer> result = offerTable.getAll();
        List<Offer> filterResult = new ArrayList<>(result.size());

        for (int i = 0; i < result.size(); i++) {
            Offer offer = result.get(i);
            if (offer.getSalary() > minSalary) {
                filterResult.add(offer);
            }
        }

        return filterResult.toArray(new Offer[0]);
    }

    public Offer[] searchOfferByName(String keyWord) throws IOException, ClassNotFoundException {
        List<Offer> result = offerTable.getAll();
        List<Offer> filterResult = new ArrayList<>(result.size());

        for (int i = 0; i < result.size(); i++) {
            Offer offer = result.get(i);
            if (offer.getJobName().matches(".*?" + keyWord + ".*")) {
                filterResult.add(offer);
            }
        }

        return filterResult.toArray(new Offer[0]);
    }

    public void deleteOffer(String user, String id) throws IOException, ClassNotFoundException {
        MultiIndex index = employerOfferIndex.select(user);
        if (index == null || !index.hasValue(id)) {
            return;
        }

        index.removeValue(id);
        employerOfferIndex.insertOrUpdate(index);

        MultiIndex offerApplication = offerApplicationIndex.select(id);
        if (offerApplication == null) {
            return;
        }

        String[] applications = offerApplication.getAllValues();
        for (String application : applications) {
            offerApplicationTable.delete(application);
        }

        offerApplicationIndex.insertOrUpdate(offerApplication);
        offerTable.delete(id);
    }

    public Offer getOffer(String id) throws IOException, ClassNotFoundException {
        return offerTable.select(id);
    }

    public Offer[] getOffer(int page, int size) throws IOException, ClassNotFoundException {
        List<Offer> offerList = offerTable.getAll();
        Offer[] allOffer = new Offer[size];
        offerList.toArray(allOffer);

        return Arrays.copyOfRange(allOffer, page * size, size);
    }

    public Offer[] getAllOffer() throws IOException, ClassNotFoundException {
        List<Offer> offerList = offerTable.getAll();
        Offer[] allOffer = new Offer[offerList.size()];

        return offerList.toArray(allOffer);
    }
}
