package net.lensfrex.disillusion.web.service.student;

import net.lensfrex.disillusion.dao.DaoBase;
import net.lensfrex.disillusion.dao.OfferApplicationTable;
import net.lensfrex.disillusion.dao.OfferTable;
import net.lensfrex.disillusion.dao.index.application.EmployerApplicationIndex;
import net.lensfrex.disillusion.dao.index.application.OfferApplicationIndex;
import net.lensfrex.disillusion.dao.index.application.StudentApplicationIndex;
import net.lensfrex.disillusion.data.ArrayList;
import net.lensfrex.disillusion.data.List;
import net.lensfrex.disillusion.model.OfferApplication;
import net.lensfrex.disillusion.model.MultiIndex;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.employer.Offer;
import net.lensfrex.disillusion.web.request.OfferApplicationRequest;
import net.lensfrex.disillusion.web.service.user.UserService;
import org.apache.commons.lang.RandomStringUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;

@Service
public class ApplicationService {
    private final OfferTable offerTable;

    private final OfferApplicationIndex offerApplicationIndex;
    private final StudentApplicationIndex studentApplicationIndex;
    private final EmployerApplicationIndex employerApplicationIndex;

    private final OfferApplicationTable offerApplicationTable;

    private final UserService userService;

    public ApplicationService(OfferTable offerTable,
                              OfferApplicationIndex offerApplicationIndex,
                              StudentApplicationIndex studentApplicationIndex,
                              EmployerApplicationIndex employerApplicationIndex,
                              OfferApplicationTable offerApplicationTable,
                              UserService userService) {

        this.offerTable = offerTable;

        this.offerApplicationIndex = offerApplicationIndex;
        this.studentApplicationIndex = studentApplicationIndex;
        this.employerApplicationIndex = employerApplicationIndex;
        this.offerApplicationTable = offerApplicationTable;
        this.userService = userService;
    }

    public void addApplication(String user, OfferApplicationRequest request) throws IOException, ClassNotFoundException {
        Offer offer = offerTable.select(request.getOfferId());
        if (offer == null) {
            return;
        }

        String employer = offer.getPublisherId();
        User employerUser = userService.getAccount(employer);
        if (employerUser == null) {
            return;
        }

        OfferApplication application = packageApplication(user, RandomStringUtils.randomAlphanumeric(8), employer, request);

        offerApplicationTable.insertOrUpdate(application);

        addRelationIndex(offerApplicationIndex, offer.getId(), application.getId());
        addRelationIndex(studentApplicationIndex, user, application.getId());
        addRelationIndex(employerApplicationIndex, employer, offer.getId());
    }

    private OfferApplication packageApplication(String user, String applicationId, String target,
                                                OfferApplicationRequest request) {

        OfferApplication application = new OfferApplication();

        application.setStatus(OfferApplication.Status.WAITING);

        application.setOfferId(request.getOfferId());
        application.setResumeId(request.getResumeId());

        application.setPublisherId(user);
        application.setId(applicationId);
        application.setTargetId(target);

        return application;
    }

    private void addRelationIndex(DaoBase<MultiIndex> index, String id, String value)
            throws IOException, ClassNotFoundException {

        MultiIndex multiIndex = index.select(id);
        if (multiIndex == null) {
            multiIndex = new MultiIndex(id);
            multiIndex.addValue(value);
        } else {
            multiIndex.addValue(value);
        }

        index.insertOrUpdate(multiIndex);
    }

    private void removeRelationIndex(DaoBase<MultiIndex> index, String id, String value)
            throws IOException, ClassNotFoundException {

        MultiIndex multiIndex = index.select(id);
        if (multiIndex == null || !multiIndex.hasValue(id)) {
            return;
        }

        multiIndex.removeValue(value);
        index.insertOrUpdate(multiIndex);
    }

    public OfferApplication[] getAllApplicationByUser(String user) throws IOException, ClassNotFoundException {
        return getFromIndex(studentApplicationIndex, user);
    }

    public OfferApplication[] getAllApplicationByTarget(String employer) throws IOException, ClassNotFoundException {
        return getFromIndex(employerApplicationIndex, employer);
    }

    public OfferApplication[] getAllApplicationByOfferId(String offerId) throws IOException, ClassNotFoundException {
        return getFromIndex(offerApplicationIndex, offerId);
    }

    private OfferApplication[] getFromIndex(DaoBase<MultiIndex> indexDao, String queryKey)
            throws IOException, ClassNotFoundException {

        MultiIndex index = indexDao.select(queryKey);
        if (index == null) {
            return new OfferApplication[0];
        }

        String[] ids = index.getAllValues();
        List<OfferApplication> applications = new ArrayList<>(ids.length);
        for (String id : ids) {
            applications.add(offerApplicationTable.select(id));
        }

        return applications.toArray(new OfferApplication[applications.size()]);
    }

    public void deleteOfferApplication(String user, String id)
            throws IOException, ClassNotFoundException {

        removeRelationIndex(offerApplicationIndex, user, id);
        removeRelationIndex(employerApplicationIndex, user, id);
        removeRelationIndex(studentApplicationIndex, user, id);

        offerApplicationTable.delete(id);
    }

    public OfferApplication getOfferApplication(String id) throws IOException, ClassNotFoundException {
        return offerApplicationTable.select(id);
    }

    public void changeApplicationStatus(String id, OfferApplication.Status status)
            throws IOException, ClassNotFoundException {

        OfferApplication application = getOfferApplication(id);
        if (application == null) {
            return;
        }

        application.setStatus(status);

        offerApplicationTable.insertOrUpdate(application);
    }
}
