package net.lensfrex.disillusion.web.service.student;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import net.lensfrex.disillusion.dao.ResumeTable;
import net.lensfrex.disillusion.dao.index.StudentResumeIndex;
import net.lensfrex.disillusion.data.ArrayList;
import net.lensfrex.disillusion.data.List;
import net.lensfrex.disillusion.model.MultiIndex;
import net.lensfrex.disillusion.model.student.Resume;
import net.lensfrex.disillusion.web.request.ResumeRequest;
import org.apache.commons.lang.RandomStringUtils;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

@Service
public class ResumeService {

    private final StudentResumeIndex studentResumeIndex;
    private final ResumeTable resumeTable;

    private static final String resumeFileLocationBase = "database/files/resume";
    private static final File baseDir = new File(resumeFileLocationBase);

    public ResumeService(StudentResumeIndex studentResumeIndex,
                         ResumeTable resumeTable) {

        this.studentResumeIndex = studentResumeIndex;
        this.resumeTable = resumeTable;
    }

    public void addResume(String user, ResumeRequest request) throws IOException, ClassNotFoundException {
        String resumeId = RandomStringUtils.randomAlphanumeric(8);

        Resume resume = new Resume();
        resume.setOwnerId(user);
        resume.setId(resumeId);
        resume.setTitle(request.getTitle());
        resume.setRemark(request.getRemark());
        resume.setResumeFileId(resumeId);

        resumeTable.insertOrUpdate(resume);

        if (!baseDir.exists()) {
            baseDir.mkdirs();
        }

        Files.writeString(Paths.get(resumeFileLocationBase, resumeId),
                request.getResumeFileBase64(),
                StandardOpenOption.CREATE, StandardOpenOption.WRITE);

        MultiIndex multiIndex = studentResumeIndex.select(user);
        if (multiIndex == null) {
            multiIndex = new MultiIndex(user);
            multiIndex.addValue(resume.getId());
        } else {
            multiIndex.addValue(resume.getId());
        }

        studentResumeIndex.insertOrUpdate(multiIndex);
    }

    public void updateResume(String user, String resumeId, ResumeRequest request) throws IOException, ClassNotFoundException {
        MultiIndex multiIndex = studentResumeIndex.select(user);
        if (multiIndex == null || !multiIndex.hasValue(resumeId)) {
            return;
        }

        Resume resume = new Resume();
        resume.setOwnerId(user);
        resume.setId(resumeId);
        resume.setTitle(request.getTitle());
        resume.setRemark(request.getRemark());
        resume.setResumeFileId(resumeId);

        updateResume(resume, request.getResumeFileBase64());
    }

    public void updateResume(Resume resume, String resumeFileData) throws IOException, ClassNotFoundException {
        if (resumeFileData != null) {
            Files.writeString(Paths.get(resumeFileLocationBase, resume.getId()), resumeFileData, StandardOpenOption.WRITE);
        }

        resumeTable.insertOrUpdate(resume);
    }

    public Resume[] getAllResume(String user) throws IOException, ClassNotFoundException {
        MultiIndex index = studentResumeIndex.select(user);
        if (index == null) {
            return new Resume[0];
        }

        String[] ids = index.getAllValues();
        List<Resume> resumes = new ArrayList<>(ids.length);
        for (String id : ids) {
            resumes.add(resumeTable.select(id));
        }

        return resumes.toArray(new Resume[resumes.size()]);
    }

    public void deleteResume(String user, String id) throws IOException, ClassNotFoundException {
        MultiIndex index = studentResumeIndex.select(user);
        if (index == null || !index.hasValue(id)) {
            return;
        }

        index.removeValue(id);
        studentResumeIndex.insertOrUpdate(index);
        resumeTable.delete(id);
    }

    public Resume getResume(String id) throws IOException, ClassNotFoundException {
        return resumeTable.select(id);
    }

    public String getResumeFileData(String fileId) throws IOException {
        Path file = Paths.get(resumeFileLocationBase, fileId);
        if (FileUtil.exists(file, true)) {
            return Files.readString(file);
        } else {
            return Base64.encode("文件不存在");
        }
    }
}
