package net.lensfrex.disillusion.data;

import java.io.Serial;
import java.util.Arrays;
import java.util.Objects;

/**
 * 简略版的顺序表（ArrayList），参考了Java内置的ArrayList实现
 *
 * @param <T>
 */
public class ArrayList<T> implements List<T> {
    @Serial
    private static final long serialVersionUID = 114514L;

    private static final int DEFAULT_MAX_SIZE = 10;

    private static final Object[] EMPTY_ARRAY = {};

    private int size = 0;

    private Object[] data;

    public ArrayList(int initialSize) {
        data = new Object[initialSize];
    }

    public ArrayList() {
        data = EMPTY_ARRAY;
    }

    @Override
    @SuppressWarnings("unchecked")
    public T get(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException(String.format("index %s is out of size %s", index, size));
        }

        return (T) data[index];
    }

    @Override
    public int size() {
        return this.size;
    }

    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public void add(T value) {
        if (value == null) {
            return;
        }

        if (size == data.length) {
            this.resize();
        }

        this.data[size++] = value;
    }

    @Override
    public void add(T value, int index) {
        if (value == null) {
            return;
        }

        checkIndex(index);
        if (data.length == size) {
            this.resize();
        }

        System.arraycopy(this.data, index,
                this.data, index + 1,
                size - index);

        this.data[index] = value;
        size++;
    }

    @Override
    public void set(int index, T newValue) {
        if (newValue == null) {
            return;
        }

        checkIndex(index);

        this.data[index] = newValue;
    }

    @Override
    public void remove(int index) {
        checkIndex(index);

        int numMoved = size - index - 1;
        if (numMoved > 0) {
            System.arraycopy(this.data, index + 1,
                    this.data, index, numMoved);
        }
        this.data[--size] = null;
        this.trim();
    }

    @Override
    public void remove(Object value) {
        int index = indexOf(value);
        if (index >= 0) {
            remove(index);
        }
    }

    public void trim() {
        if (size < data.length) {
            data = (size == 0) ? EMPTY_ARRAY : Arrays.copyOf(data, size);
        }
    }

    @Override
    public int indexOf(Object value) {
        for (int i = 0; i < size; i++) {
            if (Objects.equals(value, data[i])) {
                return i;
            }
        }

        return -1;
    }

    @Override
    public Object[] toArray() {
        return Arrays.copyOf(data, data.length);
    }

    @SuppressWarnings("unchecked")
    public T[] toArray(T[] array) {
        if (array.length < size) {
            return (T[]) Arrays.copyOf(data, size, array.getClass());
        }
        System.arraycopy(data, 0, array, 0, size);

        return array;
    }

    @Override
    public boolean contains(Object obj) {
        for (int i = 0; i < size; i++) {
            if (data[i].equals(obj)) {
                return true;
            }
        }

        return false;
    }

    private void checkIndex(int index) {
        if (index < 0 || index > size) {
            throw new IndexOutOfBoundsException(indexOutOfBoundsMessage(index));
        }
    }

    private String indexOutOfBoundsMessage(int index) {
        return String.format("Index %s out of %s", index, size);
    }

    private void resize() {
        int newSize = (this.size + 1) * 2;

        if (this.data != EMPTY_ARRAY) {
            this.data = Arrays.copyOf(data, newSize);
        } else {
            this.data = new Object[Math.max(newSize, DEFAULT_MAX_SIZE)];
        }
    }
}
